@@ -1,5 +1,6 @@
 /*  Joystick plugin for xmms by Tim Ferguson (timf@dgs.monash.edu.au
  *                                  http://www.dgs.monash.edu.au/~timf/) ...
+ *  14/12/2000 - patched to allow 5 or more buttons to be used (Justin Wake <justin@globalsoft.com.au>)
  *  XMMS is Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
  *  
  *  This program is free software; you can redistribute it and/or modify
@@ -38,6 +39,7 @@ void joy_about(void)
 	label = gtk_label_new(_("Joystick Control Plugin\n\n "
 				"Created by Tim Ferguson <timf@dgs.monash.edu.au>.\n "
 				"http://www.dgs.monash.edu.au/~timf/\n\n "
+				"5+ button support by Justin Wake <justin@globalsoft.com.au>\n\n "
 				"Control XMMS with one or two joysticks.\n ")
 );
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
