@@ -21,6 +21,7 @@
 #include <pthread.h>
 #include "xmms/i18n.h"
 #include "libxmms/util.h"
+#include "libxmms/titlestring.h"
 
 #ifdef HAVE_LINUX_CDROM_H
 #include <linux/cdrom.h>
@@ -95,7 +96,7 @@ static pthread_t stop_thread;
 #define STOP_UNLOCK() pthread_mutex_unlock(&stop_mutex)
 
 
-#if !defined(CDROMVOLREAD) && !(defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__))
+#if !defined(CDROMVOLREAD) && !(defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__)))
 static gint volume_left = 100, volume_right = 100;
 #endif
 
@@ -120,6 +121,7 @@ static void cdda_init(void)
 		xmms_cfg_read_string(cfgfile, "CDDA", "device", &cdda_cfg.device);
 		xmms_cfg_read_string(cfgfile, "CDDA", "directory", &cdda_cfg.directory);
 		xmms_cfg_read_boolean(cfgfile, "CDDA", "use_oss_mixer", &cdda_cfg.use_oss_mixer);
+		xmms_cfg_read_boolean(cfgfile, "CDDA", "title_override", &cdda_cfg.title_override);
 		xmms_cfg_read_string(cfgfile, "CDDA", "name_format", &cdda_cfg.name_format);
 		xmms_cfg_read_boolean(cfgfile, "CDDA", "use_cddb", &cdda_cfg.use_cddb);
 		xmms_cfg_read_string(cfgfile, "CDDA", "cddb_server", &cdda_cfg.cddb_server);
@@ -138,6 +140,11 @@ static void cdda_init(void)
 		cdda_cfg.device = g_strdup("/dev/acd0c");
 	if (!cdda_cfg.directory)
 		cdda_cfg.directory = g_strdup("/cdrom");
+# elif defined __OpenBSD__
+	if (!cdda_cfg.device)
+		cdda_cfg.device = g_strdup("/dev/cd0c");
+	if (!cdda_cfg.directory)
+		cdda_cfg.directory = g_strdup("/cdrom");
 # else
 	if (!cdda_cfg.device)
 		cdda_cfg.device = g_strdup("/vol/dev/aliases/cdrom0");
@@ -183,11 +190,16 @@ static int is_our_file(char *filename)
 
 static gboolean is_mounted(gchar * device_name)
 {
-#ifdef HAVE_MNTENT_H
-	FILE *mounts;
-	struct mntent *mnt;
-	char devname[256];
-	struct stat st;
+#if defined(HAVE_MNTENT_H) || defined(HAVE_GETMNTINFO)
+  	char devname[256];
+  	struct stat st;
+#if defined(HAVE_MNTENT_H)
+ 	FILE *mounts;
+ 	struct mntent *mnt;
+#elif defined(HAVE_GETMNTINFO)
+ 	struct statfs *fsp;
+ 	int entries;
+#endif
 
 	if (lstat(device_name, &st) < 0)
 		return -1;
@@ -197,6 +209,7 @@ static gboolean is_mounted(gchar * device_name)
 	else
 		strncpy(devname, device_name, 256);
 
+#if defined(HAVE_MNTENT_H)		
 	if ((mounts = setmntent(MOUNTED, "r")) == NULL)
 		return TRUE;
 
@@ -209,6 +222,18 @@ static gboolean is_mounted(gchar * device_name)
 		}
 	}
 	endmntent(mounts);
+#elif defined(HAVE_GETMNTINFO)
+ 	entries = getmntinfo(&fsp, MNT_NOWAIT);
+ 	if (entries < 0)
+ 		return NULL;
+	
+	while (entries-- > 0)
+	{
+		if (!strcmp(fsp->f_mntfromname, devname))
+			return TRUE;
+		fsp++;
+	}
+#endif                            
 #endif
 	return FALSE;
 }
@@ -283,6 +308,7 @@ static char * cdda_get_title(cdda_disc_toc_t *toc, gint track)
 	static guint32 cached_id;
 	static cdinfo_t cdinfo;
 	static pthread_mutex_t title_mutex = PTHREAD_MUTEX_INITIALIZER;
+	TitleInput *input;
 	guint32 disc_id;
 	gchar *title;
 
@@ -312,7 +338,19 @@ static char * cdda_get_title(cdda_disc_toc_t *toc, gint track)
 	}
 	pthread_mutex_unlock(&title_mutex);
 	
-	title = cdda_cdinfo_get_name(&cdinfo, track, cdda_cfg.name_format);
+	XMMS_NEW_TITLEINPUT(input);
+	cdda_cdinfo_get(&cdinfo, track, &input->performer, &input->album_name,
+			&input->track_name);
+	input->track_number = track;
+	input->file_name = input->file_path = 
+	    g_strdup_printf(_("CD Audio Track %02u"), track);
+	input->file_ext = "cda";
+	title =  xmms_get_titlestring(cdda_cfg.title_override ?
+				      cdda_cfg.name_format :
+				      xmms_get_gentitle_format(), input);
+	g_free(input->file_name);
+	g_free(input);
+
 	if (!title)
 		title = g_strdup_printf(_("CD Audio Track %02u"), track);
 	return title;
@@ -374,7 +412,7 @@ static void stop(void)
 		pthread_attr_setdetachstate(attr, PTHREAD_CREATE_DETACHED);
 	}
 	
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 /*  	ioctl(cdda_fd, CDIOCSTOP, 0); */
 	ioctl(cdda_fd, CDIOCPAUSE, 0);
 #else
@@ -392,7 +430,7 @@ static void stop(void)
 
 static void cdda_pause(short p)
 {
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 	ioctl(cdda_fd, p ? CDIOCPAUSE : CDIOCRESUME);
 #else
 	ioctl(cdda_fd, p ? CDROMPAUSE : CDROMRESUME);
@@ -404,7 +442,7 @@ static void seek(int time)
 {
 	struct cdda_msf *end, start;
 
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 	struct ioc_play_msf msf;
 #else
 	struct cdrom_msf msf;
@@ -419,7 +457,7 @@ static void seek(int time)
 	else
 		end = &cd_toc.track[track + 1];
 
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 	msf.start_m = start.minute;
 	msf.start_s = start.second;
 	msf.start_f = start.frame;
@@ -443,7 +481,7 @@ static void seek(int time)
 static int get_time(void)
 {
 	gint frame, start_frame, length;
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 	struct ioc_read_subchannel subchnl;
 	struct cd_sub_channel_info subinfo;
 
@@ -505,9 +543,9 @@ static void get_song_info(char *filename, char **title, int *len)
 
 static void get_volume(int *l, int *r)
 {
+#if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_MACHINE_SOUNDCARD_H)
 	int fd, devs, cmd, v;
 
-#if defined(HAVE_SYS_SOUNDCARD_H) || defined(HAVE_MACHINE_SOUNDCARD_H)
 	if (cdda_cfg.use_oss_mixer)
 	{
 		fd = open(DEV_MIXER, O_RDONLY);
@@ -534,7 +572,7 @@ static void get_volume(int *l, int *r)
 	if (!cdda_cfg.use_oss_mixer)
 	{
 
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 		struct ioc_vol vol;
 
 		if (cdda_fd != -1)
@@ -562,7 +600,7 @@ static void get_volume(int *l, int *r)
 
 static void set_volume(int l, int r)
 {
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 	struct ioc_vol vol;
 #else
 	struct cdrom_volctrl vol;
@@ -595,7 +633,7 @@ static void set_volume(int l, int r)
 	{
 		if (cdda_fd != -1)
 		{
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 			vol.vol[0] = vol.vol[2] = (l * 255) / 100;
 			vol.vol[1] = vol.vol[3] = (r * 255) / 100;
 			ioctl(cdda_fd, CDIOCSETVOL, &vol);
@@ -605,7 +643,7 @@ static void set_volume(int l, int r)
 			ioctl(cdda_fd, CDROMVOLCTRL, &vol);
 #endif
 		}
-#if !defined(CDROMVOLREAD) && !(defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__))
+#if !defined(CDROMVOLREAD) && !(defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__)))
 		volume_left = l;
 		volume_right = r;
 #endif
@@ -616,9 +654,13 @@ gboolean cdda_get_toc(cdda_disc_toc_t *info)
 {
 	gboolean retv = FALSE;
 
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 	struct ioc_toc_header tochdr;
+#if defined(__OpenBSD__)
+	struct ioc_read_toc_entry tocentry;
+#else
 	struct ioc_read_toc_single_entry tocentry;
+#endif
 #else	
 	struct cdrom_tochdr tochdr;
 	struct cdrom_tocentry tocentry;
@@ -634,13 +676,27 @@ gboolean cdda_get_toc(cdda_disc_toc_t *info)
 
 	memset(info, 0, sizeof(cdda_disc_toc_t));
 
-#if defined(HAVE_SYS_CDIO_H) && defined(__FreeBSD__)
+#if defined(HAVE_SYS_CDIO_H) && (defined(__FreeBSD__) || defined(__OpenBSD__))
 	if ( ioctl(fd, CDIOREADTOCHEADER, &tochdr) )
 	    goto done;
 
 	for (i = tochdr.starting_track; i <= tochdr.ending_track; i++)
 	{		
 		tocentry.address_format = CD_MSF_FORMAT;
+
+#if defined(__OpenBSD__)
+		tocentry.starting_track = i;
+		if (ioctl(fd, CDIOREADTOCENTRYS, &tocentry))
+			goto done;
+		info->track[i].minute =
+		    tocentry.data->addr.msf.minute;
+		info->track[i].second =
+		    tocentry.data->addr.msf.second;
+		info->track[i].frame =
+		    tocentry.data->addr.msf.frame;
+		info->track[i].flags.data_track =
+		    (tocentry.data->control & 4) == 4;
+#else
 		tocentry.track = i;
 		if (ioctl(fd, CDIOREADTOCENTRY, &tocentry))
 			goto done;
@@ -651,17 +707,28 @@ gboolean cdda_get_toc(cdda_disc_toc_t *info)
 		info->track[i].frame =
 		    tocentry.entry.addr.msf.frame;
 		info->track[i].flags.data_track =
-		    tocentry.entry.control & 4 == 4;
+		    (tocentry.entry.control & 4) == 4;
+#endif
 	}
 
 	/* Get the leadout track */
-	tocentry.track = 0xAA;
 	tocentry.address_format = CD_MSF_FORMAT;
+
+#if defined(__OpenBSD__)
+	tocentry.starting_track = 0xAA;
+	if (ioctl(fd, CDIOREADTOCENTRYS, &tocentry))
+		goto done;
+	info->leadout.minute = tocentry.data->addr.msf.minute;
+	info->leadout.second = tocentry.data->addr.msf.second;
+	info->leadout.frame = tocentry.data->addr.msf.frame;
+#else
+	tocentry.track = 0xAA;
 	if (ioctl(fd, CDIOREADTOCENTRY, &tocentry))
 		goto done;
 	info->leadout.minute = tocentry.entry.addr.msf.minute;
 	info->leadout.second = tocentry.entry.addr.msf.second;
 	info->leadout.frame = tocentry.entry.addr.msf.frame;
+#endif
 	
 	info->first_track = tochdr.starting_track;
 	info->last_track = tochdr.ending_track;
