@@ -1,5 +1,6 @@
 #include "mpg123.h"
 #include "libxmms/configfile.h"
+#include "libxmms/titlestring.h"
 #include <string.h>
 #include <stdlib.h>
 #include <pthread.h>
@@ -173,7 +174,6 @@ void set_mpg123_synth_functions(struct frame *fr)
 static void init(void)
 {
 	ConfigFile *cfg;
-	gchar *filename;
 
 	mpg123_make_decode_tables(outscale);
 
@@ -189,57 +189,41 @@ static void init(void)
 	mpg123_cfg.proxy_pass = NULL;
 	mpg123_cfg.cast_title_streaming = FALSE;
 	mpg123_cfg.use_udp_channel = TRUE;
-	mpg123_cfg.use_id3 = TRUE;
+	mpg123_cfg.title_override = FALSE;
 	mpg123_cfg.disable_id3v2 = FALSE;
 	mpg123_cfg.detect_by_content = FALSE;
-#ifdef USE_3DNOW
 	mpg123_cfg.use_3dnow = 0;
-#endif
 
-	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
-	if ((cfg = xmms_cfg_open_file(filename)) != 0)
-	{
-		xmms_cfg_read_int(cfg, "MPG123", "resolution", &mpg123_cfg.resolution);
-		xmms_cfg_read_int(cfg, "MPG123", "channels", &mpg123_cfg.channels);
-		xmms_cfg_read_int(cfg, "MPG123", "downsample", &mpg123_cfg.downsample);
-		/*xmms_cfg_read_int(cfg,"MPG123","downsample_custom",&mpg123_cfg.downsample_custom); */
-		xmms_cfg_read_int(cfg, "MPG123", "http_buffer_size", &mpg123_cfg.http_buffer_size);
-		xmms_cfg_read_int(cfg, "MPG123", "http_prebuffer", &mpg123_cfg.http_prebuffer);
-		xmms_cfg_read_boolean(cfg, "MPG123", "save_http_stream", &mpg123_cfg.save_http_stream);
-		if (!xmms_cfg_read_string(cfg, "MPG123", "save_http_path", &mpg123_cfg.save_http_path))
-			mpg123_cfg.save_http_path = g_strdup(g_get_home_dir());
-		xmms_cfg_read_boolean(cfg, "MPG123", "cast_title_streaming", &mpg123_cfg.cast_title_streaming);
-		xmms_cfg_read_boolean(cfg, "MPG123", "use_udp_channel", &mpg123_cfg.use_udp_channel);
+	cfg = xmms_cfg_open_default_file();
 
-		xmms_cfg_read_boolean(cfg, "MPG123", "use_proxy", &mpg123_cfg.use_proxy);
-		if (!xmms_cfg_read_string(cfg, "MPG123", "proxy_host", &mpg123_cfg.proxy_host))
-			mpg123_cfg.proxy_host = g_strdup("localhost");
-		xmms_cfg_read_int(cfg, "MPG123", "proxy_port", &mpg123_cfg.proxy_port);
-		xmms_cfg_read_boolean(cfg, "MPG123", "proxy_use_auth", &mpg123_cfg.proxy_use_auth);
-		xmms_cfg_read_string(cfg, "MPG123", "proxy_user", &mpg123_cfg.proxy_user);
-		xmms_cfg_read_string(cfg, "MPG123", "proxy_pass", &mpg123_cfg.proxy_pass);
-		
-		xmms_cfg_read_boolean(cfg, "MPG123", "use_id3", &mpg123_cfg.use_id3);
-		xmms_cfg_read_boolean(cfg, "MPG123", "disable_id3v2", &mpg123_cfg.disable_id3v2);
-		if (!xmms_cfg_read_string(cfg, "MPG123", "id3_format", &mpg123_cfg.id3_format))
-			mpg123_cfg.id3_format = g_strdup("%1 - %2");
-		xmms_cfg_read_boolean(cfg, "MPG123", "detect_by_content", &mpg123_cfg.detect_by_content);
-
-#ifdef USE_3DNOW
-		if (!xmms_cfg_read_int(cfg, "MPG123", "use_3dnow", &mpg123_cfg.use_3dnow))
-			mpg123_cfg.use_3dnow=0;
-#endif
-		xmms_cfg_free(cfg);
-	}
-	else
-	{
-		mpg123_cfg.id3_format = g_strdup("%1 - %2");
-		mpg123_cfg.proxy_host = g_strdup("localhost");
+	xmms_cfg_read_int(cfg, "MPG123", "resolution", &mpg123_cfg.resolution);
+	xmms_cfg_read_int(cfg, "MPG123", "channels", &mpg123_cfg.channels);
+	xmms_cfg_read_int(cfg, "MPG123", "downsample", &mpg123_cfg.downsample);
+	/*xmms_cfg_read_int(cfg,"MPG123","downsample_custom",&mpg123_cfg.downsample_custom); */
+	xmms_cfg_read_int(cfg, "MPG123", "http_buffer_size", &mpg123_cfg.http_buffer_size);
+	xmms_cfg_read_int(cfg, "MPG123", "http_prebuffer", &mpg123_cfg.http_prebuffer);
+	xmms_cfg_read_boolean(cfg, "MPG123", "save_http_stream", &mpg123_cfg.save_http_stream);
+	if (!xmms_cfg_read_string(cfg, "MPG123", "save_http_path", &mpg123_cfg.save_http_path))
 		mpg123_cfg.save_http_path = g_strdup(g_get_home_dir());
-#ifdef USE_3DNOW
-		mpg123_cfg.use_3dnow = 0;
-#endif
-	}
+	xmms_cfg_read_boolean(cfg, "MPG123", "cast_title_streaming", &mpg123_cfg.cast_title_streaming);
+	xmms_cfg_read_boolean(cfg, "MPG123", "use_udp_channel", &mpg123_cfg.use_udp_channel);
+
+	xmms_cfg_read_boolean(cfg, "MPG123", "use_proxy", &mpg123_cfg.use_proxy);
+	if (!xmms_cfg_read_string(cfg, "MPG123", "proxy_host", &mpg123_cfg.proxy_host))
+		mpg123_cfg.proxy_host = g_strdup("localhost");
+	xmms_cfg_read_int(cfg, "MPG123", "proxy_port", &mpg123_cfg.proxy_port);
+	xmms_cfg_read_boolean(cfg, "MPG123", "proxy_use_auth", &mpg123_cfg.proxy_use_auth);
+	xmms_cfg_read_string(cfg, "MPG123", "proxy_user", &mpg123_cfg.proxy_user);
+	xmms_cfg_read_string(cfg, "MPG123", "proxy_pass", &mpg123_cfg.proxy_pass);
+
+	xmms_cfg_read_boolean(cfg, "MPG123", "title_override", &mpg123_cfg.title_override);
+	xmms_cfg_read_boolean(cfg, "MPG123", "disable_id3v2", &mpg123_cfg.disable_id3v2);
+	if (!xmms_cfg_read_string(cfg, "MPG123", "id3_format", &mpg123_cfg.id3_format))
+		mpg123_cfg.id3_format = g_strdup("%1 - %2");
+	xmms_cfg_read_boolean(cfg, "MPG123", "detect_by_content", &mpg123_cfg.detect_by_content);
+	xmms_cfg_read_int(cfg, "MPG123", "use_3dnow", &mpg123_cfg.use_3dnow);
+
+	xmms_cfg_free(cfg);
 }
 
 /* needed for is_our_file() */
@@ -425,7 +409,18 @@ static int is_our_file(char *filename)
 	guint16 wavid;
 
 	if (!strncasecmp(filename, "http://", 7))
-	{			/* We assume all http:// are mpeg -- why do we do that? */
+	{			/* We assume all http:// (except those ending in .ogg) are mpeg -- why do we do that? */
+		ext = strrchr(filename, '.');
+		if (ext) 
+		{
+			if (!strncasecmp(ext, ".ogg", 4)) 
+				return FALSE;
+			if (!strncasecmp(ext, ".rm", 3) || 
+			    !strncasecmp(ext, ".ra", 3)  ||
+			    !strncasecmp(ext, ".rpm", 4)  ||
+			    !strncasecmp(ext, ".ram", 4))
+				return FALSE;
+		}
 		return TRUE;
 	}
 	if(mpg123_cfg.detect_by_content)
@@ -470,15 +465,6 @@ static void play_frame(struct frame *fr)
 	}
 }
 
-/* as defined in WinAmp's Nullsoft Nitrane pllugin preferences
-   default is "%1 - %2" (Artist - Title), %% = '%' */
-enum id3_format_codes
-{
-	ID3_ARTIST = '1', ID3_TITLE, ID3_ALBUM, ID3_YEAR,
-	ID3_COMMENT, ID3_GENRE, FILE_NAME, FILE_PATH,
-	FILE_EXT
-};
-
 static const gchar *get_id3_genre(unsigned char genre_code)
 {
 	return (genre_code < GENRE_MAX)
@@ -535,137 +521,35 @@ static gchar *extname(const char *filename)
 	return ext;
 }
 
-static gchar *eval_id3_format(const char *id3_format, struct id3tag_t *id3,
-			      const char *filename)
-/* returns ID3 and filename data as specified in format
-   as in Nullsoft's Nitrane plugin v1.31b (their MPEG decoder) */
-{
-	gchar *ans, c, *base, *path, *ext;
-	guint length = 0, allocated, baselen, pathlen, extlen, tmp;
-	const size_t alloc_size = 256;	/* size of memory block allocations */
-	gboolean got_field = FALSE;
-
-	ans = g_malloc(allocated = alloc_size);
-	pathlen = strlen(path = g_dirname(filename));
-	base = g_strdup(g_basename(filename));
-	if ((ext = extname(base)) == NULL)
-	{
-		ext = "";
-		extlen = 0;
-	}
-	else
-	{
-		*(ext - 1) = '\0';
-		extlen = strlen(ext);
-	}
-	baselen = strlen(base);
-	while ((c = *id3_format++) != '\0')
-	{
-		tmp = 1;
-		if (c == '%')
-		{
-			switch (*id3_format++)
-			{
-				case 0:
-					id3_format--;	/* otherwise we'll lose terminator */
-
-				case '%':
-					ans[length] = '%';
-					break;
-				case ID3_ARTIST:
-					tmp = strlen(id3->artist);
-					if(tmp != 0)
-						got_field = TRUE;
-					strncpy(&ans[length], id3->artist, tmp);
-					break;
-				case ID3_TITLE:
-					tmp = strlen(id3->title);
-					if(tmp != 0)
-						got_field = TRUE;
-					strncpy(&ans[length], id3->title, tmp);
-					break;
-				case ID3_ALBUM:
-					tmp = strlen(id3->album);
-					if(tmp != 0)
-						got_field = TRUE;
-					strncpy(&ans[length], id3->album, tmp);
-					break;
-				case ID3_YEAR:
-					tmp = strlen(id3->year);
-					if(tmp != 0)
-						got_field = TRUE;
-					strncpy(&ans[length], id3->year, tmp);
-					break;
-				case ID3_COMMENT:
-					tmp = strlen(id3->comment);					
-					if(tmp != 0)
-						got_field = TRUE;
-					strncpy(&ans[length], id3->comment, tmp);
-					break;
-				case ID3_GENRE:					
-					tmp = strlen(id3->genre);
-					if(tmp != 0)
-						got_field = TRUE;
-					strncpy(&ans[length], id3->genre, tmp);
-					break;
-				case FILE_NAME:
-					strncpy(&ans[length], base, tmp = baselen);
-					got_field = TRUE;
-					break;
-				case FILE_PATH:
-					strncpy(&ans[length], path, tmp = pathlen);
-					got_field = TRUE;
-					break;
-				case FILE_EXT:
-					strncpy(&ans[length], ext, tmp = extlen);
-					got_field = TRUE;
-					break;
-				default:
-					ans[length] = c;
-					break;
-			}
-		}
-		else
-			ans[length] = c;
-		ans[length += tmp] = '\0';
-		if (allocated - length <= 30)
-			ans = g_realloc(ans, allocated += alloc_size);
-	}
-	ans = g_realloc(ans, length + 1);
-	if(!got_field)
-	{
-		g_free(ans);
-		ans = g_strdup(base);
-	}
-	g_free(base);
-	g_free(path);
-	return ans;
-}
-
 /*
  * Function id3v1_to_id3v2 (v1, v2)
  *
  *    Convert ID3v1 tag `v1' to ID3v2 tag `v2'.
  *
  */
-
 void mpg123_id3v1_to_id3v2(struct id3v1tag_t *v1, struct id3tag_t *v2)
 {
 	memset(v2,0,sizeof(struct id3tag_t));
 	strncpy(v2->title, v1->title, 30);
 	strncpy(v2->artist, v1->artist, 30);
 	strncpy(v2->album, v1->album, 30);
-	strncpy(v2->year, v1->year, 4);
-	strncpy(v2->comment, v1->comment, 30);
+	strncpy(v2->comment, v1->u.v1_0.comment, 30);
 	strncpy(v2->genre, get_id3_genre(v1->genre), sizeof (v2->genre));
 	mpg123_strip_spaces(v2->title, 30);
 	mpg123_strip_spaces(v2->artist, 30);
 	mpg123_strip_spaces(v2->album, 30);
-	mpg123_strip_spaces(v2->year, 4);
 	mpg123_strip_spaces(v2->comment, 30);
 	mpg123_strip_spaces(v2->genre, sizeof (v2->genre));
+	v2->year = atoi(v1->year);
+
+	/* Check for v1.1 tags. */
+	if ( v1->u.v1_1.__zero == 0 )
+	    v2->track_number = v1->u.v1_1.track_number;
+	else
+	    v2->track_number = 0;
 }
 
+
 /*
  * Function mpg123_format_song_title (tag, filename)
  *
@@ -676,15 +560,27 @@ void mpg123_id3v1_to_id3v2(struct id3v1tag_t *v1, struct id3tag_t *v2)
 gchar *mpg123_format_song_title(struct id3tag_t *tag, gchar * filename)
 {
 	gchar *ret = NULL;
+	TitleInput *input;
 
-	if (mpg123_cfg.use_id3 && tag)
+	XMMS_NEW_TITLEINPUT(input);
+
+	if (tag)
 	{
-		/*
-		 * Format according to tag.
-		 */
-		ret = eval_id3_format(mpg123_cfg.id3_format,
-				      tag, filename);
+		input->performer = tag->artist;
+		input->album_name = tag->album;
+		input->track_name = tag->title;
+		input->year = tag->year;
+		input->track_number = tag->track_number;
+		input->genre = tag->genre;
+		input->comment = tag->comment;
 	}
+	input->file_name = g_basename(filename);
+	input->file_path = filename;
+	input->file_ext = extname(filename);
+	ret = xmms_get_titlestring(mpg123_cfg.title_override ?
+				   mpg123_cfg.id3_format :
+				   xmms_get_gentitle_format(), input);
+	g_free(input);
 
 	if (!ret)
 	{
@@ -709,8 +605,8 @@ gchar *mpg123_format_song_title(struct id3tag_t *tag, gchar * filename)
 void mpg123_get_id3v2(id3_t * id3d, struct id3tag_t *tag)
 {
 	id3_frame_t *id3frm;
-	char *txt;
-	int tlen;
+	gchar *txt;
+	gint tlen, num;
 
 #define ID3_SET(_tid,_fld) 						\
 {									\
@@ -732,12 +628,24 @@ void mpg123_get_id3v2(id3_t * id3d, struct id3tag_t *tag)
 		tag->_fld[0] = 0;					\
 	}								\
 }
-	ID3_SET(ID3_TIT2, title);
-	ID3_SET(ID3_TPE1, artist);
-	ID3_SET(ID3_TALB, album);
-	ID3_SET(ID3_TYER, year);
-	ID3_SET(ID3_TXXX, comment);
-	ID3_SET(ID3_TCON, genre);
+
+#define ID3_SET_NUM(_tid,_fld)				\
+{							\
+	id3frm = id3_get_frame(id3d, _tid, 1);		\
+	if (id3frm) {					\
+		num = id3_get_text_number(id3frm);	\
+		tag->_fld = num >= 0 ? num : 0;		\
+	} else						\
+		tag->_fld = 0;				\
+}
+
+	ID3_SET		(ID3_TIT2, title);
+	ID3_SET		(ID3_TPE1, artist);
+	ID3_SET		(ID3_TALB, album);
+	ID3_SET_NUM	(ID3_TYER, year);
+	ID3_SET_NUM	(ID3_TRCK, track_number);
+	ID3_SET		(ID3_TXXX, comment);
+	ID3_SET		(ID3_TCON, genre);
 }
 
 /*
@@ -1088,6 +996,7 @@ static void *decode_loop(void *arg)
 	if (output_opened && !audio_error)
 		mpg123_ip.output->close_audio();
 	g_free(mpg123_pcm_sample);
+	g_free(filename);
 	pthread_exit(NULL);
 }
 
@@ -1105,7 +1014,7 @@ static void play_file(char *filename)
 	audio_error = FALSE;
 	have_xing_header = FALSE;
 
-	pthread_create(&decode_thread, NULL, decode_loop, filename);
+	pthread_create(&decode_thread, NULL, decode_loop, g_strdup(filename));
 }
 
 static void stop(void)
