@@ -418,19 +418,19 @@ gint timeout_func(gpointer data)
 		}
 
 		new_pos = xmms_remote_get_playlist_pos(xmms_session);
-		len = xmms_remote_get_playlist_time(xmms_session, new_pos);
 		if (tooltips != NULL)
 		{
-			if (len != -1 && pl_pos != new_pos)
+			if (pl_pos != new_pos)
 			{
-				tip = g_strdup_printf("%d. %s", new_pos+1, xmms_remote_get_playlist_title(xmms_session, new_pos));
+				char *title = xmms_remote_get_playlist_title(xmms_session, new_pos);
+				if (!title)
+					title = g_strdup("");
+				tip = g_strdup_printf("%d. %s", new_pos+1, title);
+				g_free(title);
 				gtk_tooltips_set_tip(tooltips, window, tip, NULL);
 				g_free(tip);
 				pl_pos = new_pos;
 			}
-
-			if (pl_pos != -1 && len == -1)
-				gtk_tooltips_set_tip(tooltips, window, NULL, NULL);
 		}
 
 		playing = xmms_remote_is_playing(xmms_session);
@@ -443,8 +443,8 @@ gint timeout_func(gpointer data)
 		}
 		else if (playing)
 		{
-			seekslider_visible = TRUE;
-			if (len == -1 && seekslider_visible)
+			len = xmms_remote_get_playlist_time(xmms_session, new_pos);
+			if (len == -1)
 			{
 				seekslider_visible = FALSE;
 				seekslider_dragging = FALSE;
@@ -453,6 +453,7 @@ gint timeout_func(gpointer data)
 			}
 			else if (!seekslider_dragging)
 			{
+				seekslider_visible = TRUE;
 				pos = xmms_remote_get_output_time(xmms_session);
 				if (len != 0)
 					new_pos = (pos * SEEKSLIDER_MAX) / len;
@@ -507,14 +508,6 @@ void init(void)
 	gint i, w, h;
 	GdkGC *mask_gc;
 
-	volslider_dragging = FALSE;
-	volslider_pos = 0;
-
-	seekslider_visible = FALSE;
-	seekslider_dragging = FALSE;
-	seekslider_pos = -1;
-	seekslider_drag_offset = 0;
-
 	for (i = 0; i < NUM_BUTTONS; i++)
 		button_list = g_list_append(button_list, &buttons[i]);
 
@@ -727,3 +720,19 @@ int main(int argc, char **argv)
 	gtk_main();
 	return 0;
 }
+
+
+#if defined(USE_DMALLOC)
+
+/*
+ * When allocating via, say, g_strdup the wrapper macro must free the
+ * previously allocated string through the original g_free function.
+ */
+void g_free_orig (gpointer mem)
+{
+#undef g_free
+	void g_free (gpointer);
+	g_free (mem);
+}
+
+#endif
