@@ -25,14 +25,14 @@ typedef struct
 	gint *nx, *ny;
 	gint *sx, *sy;
 	gint barx, bary;
-	gint active;
+	gint active, base;
 	void (*handler) (gint item);
 }
 PlaylistPopup;
 
-PlaylistPopup *popup = NULL;
+static PlaylistPopup *popup = NULL;
 
-void playlist_popup_draw(PlaylistPopup * popup)
+static void playlist_popup_draw(PlaylistPopup * popup)
 {
 	gint i;
 
@@ -65,19 +65,19 @@ void playlist_popup_destroy(void)
 		g_free(popup->ny);
 		g_free(popup->sx);
 		g_free(popup->sy);
-		if (popup->handler)
-			popup->handler(popup->active);
+		if (popup->handler && popup->active != -1)
+			popup->handler(popup->active + popup->base);
 		g_free(popup);
 		popup = NULL;
 	}
 }
 
-void playlist_popup_expose(GtkWidget * widget, GdkEvent * event, gpointer callback_data)
+static void playlist_popup_expose(GtkWidget * widget, GdkEvent * event, gpointer callback_data)
 {
 	playlist_popup_draw(popup);
 }
 
-void playlist_popup_motion(GtkWidget * widget, GdkEventMotion * event, gpointer callback_data)
+static void playlist_popup_motion(GtkWidget * widget, GdkEventMotion * event, gpointer callback_data)
 {
 	gint active;
 
@@ -97,13 +97,13 @@ void playlist_popup_motion(GtkWidget * widget, GdkEventMotion * event, gpointer
 	}
 }
 
-void playlist_popup_release(GtkWidget * widget, GdkEventButton * event, gpointer callback_data)
+static void playlist_popup_release(GtkWidget * widget, GdkEventButton * event, gpointer callback_data)
 {
 	playlist_popup_destroy();
 }
 
 void playlist_popup(gint x, gint y, gint num_items, gint * nx, gint * ny, gint * sx, gint * sy,
-		    gint barx, gint bary, void (*handler) (gint item))
+		    gint barx, gint bary, gint base, void (*handler) (gint item))
 {
 	if (popup)
 		playlist_popup_destroy();
@@ -121,6 +121,7 @@ void playlist_popup(gint x, gint y, gint num_items, gint * nx, gint * ny, gint *
 	popup->bary = bary;
 	popup->handler = handler;
 	popup->active = num_items - 1;
+	popup->base = base;
 	popup->window = gtk_window_new(GTK_WINDOW_POPUP);
 	gtk_widget_set_app_paintable(popup->window, TRUE);
 	gtk_widget_set_events(popup->window, GDK_BUTTON_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_EXPOSURE_MASK);
@@ -131,12 +132,11 @@ void playlist_popup(gint x, gint y, gint num_items, gint * nx, gint * ny, gint *
 	gtk_signal_connect(GTK_OBJECT(popup->window), "expose_event", GTK_SIGNAL_FUNC(playlist_popup_expose), NULL);
 	gtk_signal_connect(GTK_OBJECT(popup->window), "motion_notify_event", GTK_SIGNAL_FUNC(playlist_popup_motion), NULL);
 	gtk_signal_connect(GTK_OBJECT(popup->window), "button_release_event", GTK_SIGNAL_FUNC(playlist_popup_release), NULL);
-	gtk_widget_show(popup->window);
 	gdk_window_move(popup->window->window, x - 1, y - 1);
+	gtk_widget_show(popup->window);
 	gdk_window_raise(popup->window->window);
 	gdk_flush();
 	playlist_popup_draw(popup);
 	gdk_pointer_grab(popup->window->window, FALSE, GDK_BUTTON_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, GDK_NONE, GDK_NONE, GDK_CURRENT_TIME);
 	gdk_flush();
-
 }
