@@ -33,6 +33,10 @@
 
 #include <ctype.h>
 
+#ifndef HAVE_MKDTEMP
+char* mkdtemp(char* path);
+#endif
+
 Skin *skin;
 
 static const gint skin_default_viscolor[24][3] =
@@ -65,6 +69,8 @@ static const gint skin_default_viscolor[24][3] =
 
 static void setup_skin_masks(void)
 {
+	if (cfg.show_wm_decorations)
+		return;
 	if (cfg.player_visible)
 	{
 		gtk_widget_shape_combine_mask(mainwin, skin_get_mask(SKIN_MASK_MAIN, cfg.doublesize, cfg.player_shaded), 0, 0);
@@ -507,9 +513,93 @@ void free_skin(void)
 	}
 }
 
+static void skin_load_pixmaps(const char *path)
+{
+	load_skin_pixmap(&skin->main, path, "main.bmp");
+	load_skin_pixmap(&skin->cbuttons, path, "cbuttons.bmp");
+	load_skin_pixmap(&skin->titlebar, path, "titlebar.bmp");
+	load_skin_pixmap(&skin->shufrep, path, "shufrep.bmp");
+	load_skin_pixmap(&skin->text, path, "text.bmp");
+	skin_get_textcolors(skin->text.pixmap, skin->textbg, skin->textfg);
+	load_skin_pixmap(&skin->volume, path, "volume.bmp");
+	load_skin_pixmap(&skin->balance, path, "balance.bmp");
+	if (skin->balance.pixmap == NULL)
+		load_skin_pixmap(&skin->balance, path, "volume.bmp");
+	load_skin_pixmap(&skin->monostereo, path, "monoster.bmp");
+	load_skin_pixmap(&skin->playpause, path, "playpaus.bmp");
+	load_skin_pixmap(&skin->numbers, path, "nums_ex.bmp");
+	if (skin->numbers.pixmap == NULL)
+	{
+		load_skin_pixmap(&skin->numbers, path, "numbers.bmp");
+		skin_numbers_generate_dash(&skin->numbers);
+	}
+	load_skin_pixmap(&skin->posbar, path, "posbar.bmp");
+	load_skin_pixmap(&skin->pledit, path, "pledit.bmp");
+	load_skin_pixmap(&skin->eqmain, path, "eqmain.bmp");
+	load_skin_pixmap(&skin->eq_ex, path, "eq_ex.bmp");
+	skin->pledit_normal = load_skin_color(path, "pledit.txt", "text", "normal");
+	skin->pledit_current = load_skin_color(path, "pledit.txt", "text", "current");
+	skin->pledit_normalbg = load_skin_color(path, "pledit.txt", "text", "normalbg");
+	skin->pledit_selectedbg = load_skin_color(path, "pledit.txt", "text", "selectedbg");
+	skin->mask_main = skin_create_transparent_mask(path, "region.txt", "Normal", mainwin->window, 275, 116, FALSE);
+	skin->mask_main_ds = skin_create_transparent_mask(path, "region.txt", "Normal", mainwin->window, 550, 232, TRUE);
+	skin->mask_shade = skin_create_transparent_mask(path, "region.txt", "WindowShade", mainwin->window, 275, 14, FALSE);
+	skin->mask_shade_ds = skin_create_transparent_mask(path, "region.txt", "WindowShade", mainwin->window, 550, 28, TRUE);
+	skin->mask_eq = skin_create_transparent_mask(path, "region.txt", "Equalizer", equalizerwin->window, 275, 116, FALSE);
+	skin->mask_eq_ds = skin_create_transparent_mask(path, "region.txt", "Equalizer", equalizerwin->window, 550, 232, TRUE);
+	skin->mask_eq_shade = skin_create_transparent_mask(path, "region.txt", "EqualizerWS", equalizerwin->window, 275, 14, FALSE);
+	skin->mask_eq_shade_ds = skin_create_transparent_mask(path, "region.txt", "EqualizerWS", equalizerwin->window, 550, 28, TRUE);
+	
+	load_skin_viscolor(path, "viscolor.txt");
+}
+
+static char * skin_decompress_skin(const char* path)
+{
+	char *tmp = NULL, *tempdir, *unzip, *tar, *ending;
+
+	unzip = getenv("UNZIPCMD");
+	if (!unzip)
+		unzip = "unzip";
+	tar = getenv("TARCMD");
+	if (!tar)
+		tar = "tar";
+
+	tempdir = g_strconcat(g_get_tmp_dir(), "/xmmsskin.XXXXXXXX", NULL);
+	if (!mkdtemp(tempdir))
+	{
+		g_free(tempdir);
+		g_message("Failed to create temporary directory: %s.  Unable to load skin.", strerror(errno));
+		return NULL;
+	}
+
+	ending = strrchr(path, '.');
+
+	if (!strcasecmp(ending, ".zip") || !strcasecmp(ending, ".wsz"))
+	{
+		tmp = g_strdup_printf("%s >/dev/null -o -j \"%s\" -d %s", unzip, path, tempdir);
+	}
+	if (!strcasecmp(ending, ".tgz") || !strcasecmp(ending, ".gz"))
+	{
+		tmp = g_strdup_printf("%s >/dev/null xzf \"%s\" -C %s", tar, path, tempdir);
+	}
+	if (!strcasecmp(ending, ".bz2"))
+	{
+		tmp = g_strdup_printf("%s >/dev/null xIf \"%s\" -C %s", tar, path, tempdir);
+	}
+	if (!strcasecmp(ending, ".tar"))
+	{
+		tmp = g_strdup_printf("%s >/dev/null xf \"%s\" -C %s", tar, path, tempdir);
+	}
+
+	system(tmp);
+	g_free(tmp);
+	return tempdir;
+}
+	
+
 static void _load_skin(const gchar * path, gboolean force)
 {
-	char *ending, *tmp = NULL, *tempdir, *unzip, *tar;
+	char *ending;
 
 	if (!force)
 	{
@@ -522,90 +612,25 @@ static void _load_skin(const gchar * path, gboolean force)
 	}
 	if (path)
 	{
-		if (strncmp(path, g_get_tmp_dir(), strlen(g_get_tmp_dir())))
-		{
-			skin->path = g_realloc(skin->path, strlen(path) + 1);
-			strcpy(skin->path, path);
-		}
+		skin->path = g_realloc(skin->path, strlen(path) + 1);
+		strcpy(skin->path, path);
+
 		ending = strrchr(path, '.');
-		if (ending)
+		if (ending &&
+		    (!strcasecmp(ending, ".zip") || !strcasecmp(ending, ".wsz") ||
+		     !strcasecmp(ending, ".tgz") || !strcasecmp(ending, ".gz") ||
+		     !strcasecmp(ending, ".bz2") || !strcasecmp(ending, ".tar")))
 		{
-			if (!strcasecmp(ending, ".zip") || !strcasecmp(ending, ".wsz") ||
-			    !strcasecmp(ending, ".tgz") || !strcasecmp(ending, ".gz") ||
-			    !strcasecmp(ending, ".bz2") || !strcasecmp(ending, ".tar"))
+			char *cpath = skin_decompress_skin(path);
+			if (cpath != NULL)
 			{
-
-				tempdir = tempnam(NULL, NULL);
-
-				unzip = getenv("UNZIPCMD");
-				if (!unzip)
-					unzip = "unzip";
-				tar = getenv("TARCMD");
-				if (!tar)
-					tar = "tar";
-				if (!strcasecmp(ending, ".zip") || !strcasecmp(ending, ".wsz"))
-				{
-					tmp = g_strdup_printf("%s >/dev/null -o -j \"%s\" -d %s", unzip, path, tempdir);
-				}
-				if (!strcasecmp(ending, ".tgz") || !strcasecmp(ending, ".gz"))
-				{
-					tmp = g_strdup_printf("%s >/dev/null xzf \"%s\" -C %s", tar, path, tempdir);
-					mkdir(tempdir, 0755);	/* Permissions: 755 */
-				}
-				if (!strcasecmp(ending, ".bz2"))
-				{
-					tmp = g_strdup_printf("%s >/dev/null xIf \"%s\" -C %s", tar, path, tempdir);
-					mkdir(tempdir, 0755);	/* Permissions: 755 */
-				}
-				if (!strcasecmp(ending, ".tar"))
-				{
-					tmp = g_strdup_printf("%s >/dev/null xf \"%s\" -C %s", tar, path, tempdir);
-					mkdir(tempdir, 0755);	/* Permissions: 755 */
-				}
-				system(tmp);
-				g_free(tmp);
-				load_skin(tempdir);
-				del_directory(tempdir);
-				free(tempdir);
-				return;
+				skin_load_pixmaps(cpath);
+				del_directory(cpath);
+				g_free(cpath);
 			}
 		}
-		load_skin_pixmap(&skin->main, path, "main.bmp");
-		load_skin_pixmap(&skin->cbuttons, path, "cbuttons.bmp");
-		load_skin_pixmap(&skin->titlebar, path, "titlebar.bmp");
-		load_skin_pixmap(&skin->shufrep, path, "shufrep.bmp");
-		load_skin_pixmap(&skin->text, path, "text.bmp");
-		skin_get_textcolors(skin->text.pixmap, skin->textbg, skin->textfg);
-		load_skin_pixmap(&skin->volume, path, "volume.bmp");
-		load_skin_pixmap(&skin->balance, path, "balance.bmp");
-		if (skin->balance.pixmap == NULL)
-			load_skin_pixmap(&skin->balance, path, "volume.bmp");
-		load_skin_pixmap(&skin->monostereo, path, "monoster.bmp");
-		load_skin_pixmap(&skin->playpause, path, "playpaus.bmp");
-		load_skin_pixmap(&skin->numbers, path, "nums_ex.bmp");
-		if (skin->numbers.pixmap == NULL)
-		{
-			load_skin_pixmap(&skin->numbers, path, "numbers.bmp");
-			skin_numbers_generate_dash(&skin->numbers);
-		}
-		load_skin_pixmap(&skin->posbar, path, "posbar.bmp");
-		load_skin_pixmap(&skin->pledit, path, "pledit.bmp");
-		load_skin_pixmap(&skin->eqmain, path, "eqmain.bmp");
-		load_skin_pixmap(&skin->eq_ex, path, "eq_ex.bmp");
-		skin->pledit_normal = load_skin_color(path, "pledit.txt", "text", "normal");
-		skin->pledit_current = load_skin_color(path, "pledit.txt", "text", "current");
-		skin->pledit_normalbg = load_skin_color(path, "pledit.txt", "text", "normalbg");
-		skin->pledit_selectedbg = load_skin_color(path, "pledit.txt", "text", "selectedbg");
-		skin->mask_main = skin_create_transparent_mask(path, "region.txt", "Normal", mainwin->window, 275, 116, FALSE);
-		skin->mask_main_ds = skin_create_transparent_mask(path, "region.txt", "Normal", mainwin->window, 550, 232, TRUE);
-		skin->mask_shade = skin_create_transparent_mask(path, "region.txt", "WindowShade", mainwin->window, 275, 14, FALSE);
-		skin->mask_shade_ds = skin_create_transparent_mask(path, "region.txt", "WindowShade", mainwin->window, 550, 28, TRUE);
-		skin->mask_eq = skin_create_transparent_mask(path, "region.txt", "Equalizer", equalizerwin->window, 275, 116, FALSE);
-		skin->mask_eq_ds = skin_create_transparent_mask(path, "region.txt", "Equalizer", equalizerwin->window, 550, 232, TRUE);
-		skin->mask_eq_shade = skin_create_transparent_mask(path, "region.txt", "EqualizerWS", equalizerwin->window, 275, 14, FALSE);
-		skin->mask_eq_shade_ds = skin_create_transparent_mask(path, "region.txt", "EqualizerWS", equalizerwin->window, 550, 28, TRUE);
-		
-		load_skin_viscolor(path, "viscolor.txt");
+		else
+			skin_load_pixmaps(path);
 	}
 	else
 	{
